"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const childProcess = require("child_process");
const fs = require("fs-extra");
const moment = require("moment");
const path = require("path");
const getLogger_1 = require("../logging/getLogger");
const EnvironmentUtil = require("../runtime/EnvironmentUtil");
const lifecycle = require("../runtime/lifecycle");
const config_1 = require("../settings/config");
const logger = getLogger_1.getLogger(path.basename(__filename));
const appFolder = path.resolve(process.execPath, '..');
const rootFolder = path.resolve(appFolder, '..');
const updateDotExe = path.join(rootFolder, 'Update.exe');
const exeName = `${config_1.config.name}.exe`;
const linkName = `${config_1.config.name}.lnk`;
const windowsAppData = process.env.APPDATA;
if (!windowsAppData && EnvironmentUtil.platform.IS_WINDOWS) {
    logger.error('No Windows AppData directory found.');
}
const shortcutLink = windowsAppData
    ? path.resolve(windowsAppData, 'Microsoft/Internet Explorer/Quick Launch/User Pinned/TaskBar', linkName)
    : '';
var SQUIRREL_EVENT;
(function (SQUIRREL_EVENT) {
    SQUIRREL_EVENT["CREATE_SHORTCUT"] = "--createShortcut";
    SQUIRREL_EVENT["INSTALL"] = "--squirrel-install";
    SQUIRREL_EVENT["OBSOLETE"] = "--squirrel-obsolete";
    SQUIRREL_EVENT["REMOVE_SHORTCUT"] = "--removeShortcut";
    SQUIRREL_EVENT["UNINSTALL"] = "--squirrel-uninstall";
    SQUIRREL_EVENT["UPDATE"] = "--update";
    SQUIRREL_EVENT["UPDATED"] = "--squirrel-updated";
})(SQUIRREL_EVENT || (SQUIRREL_EVENT = {}));
function spawn(command, args) {
    const commandFile = path.basename(command);
    return new Promise(resolve => {
        const spawnedProcess = childProcess
            .spawn(command, args)
            .on('error', error => logger.error(error))
            .on('close', (code, signal) => {
            if (code !== 0) {
                const exitReason = signal || `exit code ${code}`;
                logger.error(`Running command "${command}" failed: received ${exitReason}`);
            }
            resolve();
        });
        if (spawnedProcess.stdout) {
            spawnedProcess.stdout.on('data', (data) => {
                const stringifiedData = data.toString().trim();
                if (stringifiedData) {
                    logger.info(`${commandFile}: ${stringifiedData}`);
                }
            });
        }
        if (spawnedProcess.stderr) {
            spawnedProcess.stderr.on('data', (data) => {
                const stringifiedData = data.toString().trim();
                if (stringifiedData) {
                    logger.error(`${commandFile}: ${stringifiedData}`);
                }
            });
        }
    });
}
function spawnUpdate(args) {
    return __awaiter(this, void 0, void 0, function* () {
        logger.info(`Running updater ...`);
        const updateDotExeExists = fs.existsSync(updateDotExe);
        if (!updateDotExeExists) {
            logger.info(`Could not find updater in "${updateDotExe}".`);
        }
        try {
            yield spawn(updateDotExe, args);
        }
        catch (error) {
            logger.error(error);
        }
    });
}
function createStartShortcut() {
    return __awaiter(this, void 0, void 0, function* () {
        logger.info(`Creating shortcut in the start menu ...`);
        yield spawnUpdate([SQUIRREL_EVENT.CREATE_SHORTCUT, exeName, '-l=StartMenu']);
    });
}
function createDesktopShortcut() {
    return __awaiter(this, void 0, void 0, function* () {
        logger.info(`Creating shortcut on the desktop ...`);
        yield spawnUpdate([SQUIRREL_EVENT.CREATE_SHORTCUT, exeName, '-l=Desktop']);
    });
}
function removeShortcuts() {
    return __awaiter(this, void 0, void 0, function* () {
        logger.info(`Removing all shortcuts ...`);
        yield spawnUpdate([SQUIRREL_EVENT.REMOVE_SHORTCUT, exeName, '-l=Desktop,Startup,StartMenu']);
        if (shortcutLink) {
            yield fs.remove(shortcutLink);
        }
    });
}
function installUpdate() {
    return __awaiter(this, void 0, void 0, function* () {
        logger.info(`Checking for Windows updates at "${EnvironmentUtil.app.UPDATE_URL_WIN}" ...`);
        yield spawnUpdate([SQUIRREL_EVENT.UPDATE, EnvironmentUtil.app.UPDATE_URL_WIN]);
    });
}
exports.installUpdate = installUpdate;
function scheduleUpdate() {
    return __awaiter(this, void 0, void 0, function* () {
        const pluralize = (num, str) => `${num} ${str}${num === 1 ? '' : 's'}`;
        const nextCheck = moment.duration(config_1.config.squirrelUpdateInterval.DELAY).asMinutes();
        const everyCheck = moment.duration(config_1.config.squirrelUpdateInterval.INTERVAL).asHours();
        logger.info(`Scheduling Windows update to check in "${pluralize(nextCheck, 'minute')}" and every "${pluralize(everyCheck, 'hour')}" ...`);
        setTimeout(installUpdate, config_1.config.squirrelUpdateInterval.DELAY);
        setInterval(installUpdate, config_1.config.squirrelUpdateInterval.INTERVAL);
    });
}
function handleSquirrelEvent(isFirstInstance) {
    return __awaiter(this, void 0, void 0, function* () {
        const [, squirrelEvent] = process.argv;
        switch (squirrelEvent) {
            case SQUIRREL_EVENT.INSTALL: {
                yield createStartShortcut();
                yield createDesktopShortcut();
                lifecycle.quit();
                return true;
            }
            case SQUIRREL_EVENT.UPDATED: {
                lifecycle.quit();
                return true;
            }
            case SQUIRREL_EVENT.UNINSTALL: {
                yield removeShortcuts();
                lifecycle.quit();
                return true;
            }
            case SQUIRREL_EVENT.OBSOLETE: {
                lifecycle.quit();
                return true;
            }
        }
        if (!isFirstInstance) {
            return lifecycle.quit();
        }
        yield scheduleUpdate();
    });
}
exports.handleSquirrelEvent = handleSquirrelEvent;
//# sourceMappingURL=squirrel.js.map